{*******************************************************************************
Source file: ArgumentsUnit.pas
Contains the implementation of TFormArguments, a form that allows editing a
list of arguments.

Author: Niels Dekker (nd)
E-mail: dekkerware at xs4all.nl

Tested on Delphi 5, 6, and 7
********************************************************************************
Updates:
When      Who   What
yyyymmdd
20020902  nd    Created (.EXE file version 1.0.0.1)
20030716  nd    Version 1.1.0.0: gone freeware
20030728  nd    Version 1.1.1.0: Added minimum constraints to form size
20030815  nd    Removed DesignSize from DFM-file, using a tool made by
                Gerrit Beuze (ModelMaker Tools): D7 DesignSize cleaner
}

unit ArgumentsUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TFormArguments = class(TForm)
    Memo1: TMemo;
    ButtonOk: TButton;
    ButtonCancel: TButton;
    procedure Memo1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
  private
    function GetArguments: string;
    procedure SetArguments(const strNewArguments: string);
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
  public
    property Arguments: string read GetArguments write SetArguments;

  end;

procedure AddSpaceSeparatedArgumentsToList(
  const strArguments: string;
  const rList: TStrings);

var
  FormArguments: TFormArguments;

implementation

{$R *.DFM}

uses
  ShellApi;

{ TFormArguments }


procedure AddSpaceSeparatedArgumentsToList(
  const strArguments: string;
  const rList: TStrings);
var
  i: integer;
  strNewLine: string;
  bDoubleQuoted: boolean;
begin
  i := 1;
  while i <= Length(strArguments) do
  begin
    if strArguments[i] = '"' then
    begin
      bDoubleQuoted := true;
      strNewLine := '"';
      Inc(i);
    end
    else
    begin
      bDoubleQuoted := false;
      strNewLine := ''
    end;
    while i <= Length(strArguments) do
    begin
      if bDoubleQuoted then
      begin
        if strArguments[i] = '"' then
          bDoubleQuoted := false
      end
      else
      begin
        if strArguments[i] = ' ' then
          break
      end;
      strNewLine := strNewLine + strArguments[i];
      Inc(i)
    end;
    rList.Append(strNewLine);
    Inc(i)
  end
end;

procedure AddDroppedFilesToList(
  const hd: HDROP;
  const rList: TStrings);
var
  strFilename: string;
  uFile: UINT;
  uCountFiles: UINT;
  uFilenameLength: UINT;
begin
  uCountFiles := DragQueryFile(hd, $FFFFFFFF, nil, 0);
  if (uCountFiles <> 0) and (rList <> nil) then
  begin
    for uFile := 0 to uCountFiles - 1 do
    begin
      uFilenameLength := DragQueryFile(hd, uFile, nil, 0);
      SetLength(strFilename, uFilenameLength + 1);
      DragQueryFile(hd, uFile, PChar(strFilename), Length(strFilename));
      strFilename := PChar(strFilename);
      if (Pos(' ', strFilename) <> 0) and (Pos('"', strFilename) = 0) then
        strFilename :=  '"' + strFilename + '"';

      rList.Add(strFilename)
    end
  end
end;


function TFormArguments.GetArguments: string;
var
  rLines: TStrings;
  i: integer;
begin
  rLines := Memo1.Lines;
  if rLines.Count = 0 then
    result := ''
  else
  begin
    result := rLines[0];
    for i := 1 to rLines.Count - 1 do
      result := result + ' ' + rLines[i]
  end
end;

procedure TFormArguments.SetArguments(const strNewArguments: string);
begin
  Memo1.Clear();
  AddSpaceSeparatedArgumentsToList(strNewArguments, Memo1.Lines)
end;

procedure TFormArguments.Memo1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_ESCAPE) and (Shift = []) then
    ModalResult := mrCancel
  else
    if (Key = VK_RETURN) and (Shift <> []) then
    begin
      //todo joop  PeekMessage(Msg, 0, WM_SYSCHAR, WM_SYSCHAR, PM_REMOVE);
      ModalResult := mrOk
    end
end;

procedure TFormArguments.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(Handle, True);
end;

procedure TFormArguments.WMDropFiles(var Msg: TWMDropFiles);
begin
  inherited;
  try
    AddDroppedFilesToList(Msg.Drop, Memo1.Lines)
  finally
    DragFinish(Msg.Drop)
  end
end;

end.
