{*******************************************************************************
Source file: OnTheRunUnit.pas
Contains the implementation of TFormOnTheRun, the main form of OnTheRun.exe

OnTheRun.exe is designed as a WinDiff replacement for TreeComp. It allows you to
run and re-use one instance of an application by letting OnTheRun drop
your arguments on the window of that application.

Author: Niels Dekker (nd)
E-mail: dekkerware at xs4all.nl
On The Run on the web: http://www.xs4all.nl/~nd/dekkerware/ontherun/

Tested on Delphi 5, 6, and 7

Acknowledgements:
Lennert Ploeger (lsp) made a lot of great suggestions on how to improve
OnTheRun, found out how to get a "knispering" system menu, and of
course he is the creator of the application that OnTheRun wants to
"add on" in the first place: TreeComp.
The icon (OnTheRun.ico) was brought to me by Josien de Bois.
********************************************************************************
Updates:
When      Who   What
yyyymmdd
20020902  nd    Created (.EXE file version 1.0.0.1)
20020903  nd    .DFM fix: FocusControl of labels set to comboboxes
20021111  nd    Version 1.0.1.0:
                Checkbox "Pass arguments as short (8.3) path names" added.
                Adding or removing double-quotes to arguments supported.
                Drag'n'drop of files onto the Arguments combo box supported.
                Optional status bar added.  Useless Clear button removed.
20021114 nd+lsp Version 1.0.2.0:
                Added options to pass the arguments at startup of OnTheRun, and
                to minimize OnTheRun afterwards, as requested by lsp.
                Removed the "Close when done" checkbox.
                Added "..." buttons, providing access to dialog boxes that allow
                selecting the target application file and editing the arguments.
                Fix in .DFM file: StatusBar1.SimplePanel set to True, allowing
                automatic repaint, and supporting TextGrabber by Eric VanHeest.
                Workaround for Delphi providing an incomplete system menu when
                right-clicking the Windows Taskbar button, marked as "Knispering"
20021118  nd    Now status bar is emptied if the user clears the arguments.
                Typo in version info key "Add-on for:" (found by lsp) fixed.
                Removed useless "else"
20030716  nd    Version 1.1.0.0: gone freeware
20030717  nd    ComboBoxDoubleQuotes.ItemIndex added to FormStorage.
                New e-mail address: dekkerware at xs4all.nl
20030721  nd    Removed useless code in FormCreate.
                Fix: Enabled Go after browse for file.
                Bug: PutStringAtBeginOfList tried to delete the item at
                MaxInt - 1.  Fixed: delete item at iMaxSize - 1 instead!
                Note: Application crashes at closing if Count= in
                INI-file does not correspond to the number of items.
                Built in release-mode.
                Bug: Arguments and Window text might become empty after
                a Go. Fixed by selecting the corresponding combo item.
20030722  nd    Fixed previous fix by adding begin and end to if.
20030723  nd    Made the previous fix even better by checking whether
                PutStringAtBeginOfList is succesful.
20030727  nd    EnumProc callback functions made more robust:
                Magic number MAX_WINDOW_NAME_LEN (256) replaced by calls
                to Windows API GetWindowTextLength.
                No longer uses global variable FormOnTheRun.
20030728  nd    New release version (from 1.1.0.5 to 1.1.1.0):
                Made the status bar scrollable and sizable, as suggested
                by lsp.  Allowed toggling the status bar by Ctrl+B.
                Enabled maximizing the main form.
                Set default index of "double quotes" option from -1 to 0.
20030730  nd    Notepad could not open "C:\New Text Document.txt" when
                dropped by OnTheRun, because of the double quotes.  So I
                changed the interpretation of the option "Remove
                double quotes from arguments" when dropping them onto
                a window: now double quotes will be removed, even from
                arguments that contains spaces.  On the other hand,
                "Add double quotes to arguments" will even add
                double quotes to arguments that already contain
                a double quote, when dropping them onto a window.
                Also now empty strings are removed from argument list.
20030802  nd    New (beta) release version (from 1.1.1.1 to 1.1.2.0):
                Added options "1 argument only" and
                "What if dropping arguments on window fails?".
                Allowed hiding the options by "Options"-checkbox.
20030822  nd    Fixed buggy height of panes introduced in
                version 1.1.2.0 (which was a beta release anyway!).
                Use of RxLib made explicit by defining USE_RX_LIBRARY.
                Because the fix of the system menu at the taskbar
                (previously marked as "Knispering") appeared to have
                some bad side effects, it is now allowed to disable this
                fix by putting the string "INCOMPLETE_TASKBAR_MENU" into
                the application pathname or the startup directory.
                Improved coding style by enumerating custom menu items.
                Removed DesignSize from DFM-file, using a tool made by
                Gerrit Beuze (ModelMaker Tools): D7 DesignSize cleaner
20030903  nd    Added menu item "On The Run - On The Web".
                Released on http://www.xs4all.nl/~nd/dekkerware/ontherun
20030927  nd    "On The Run - On The Web" now puts URL in command.
                Location of INI file mentioned in About Box.
*******************************************************************************}
unit OnTheRunUnit;
interface

// Please remove the following directive if you don't have RxLib installed.
{$DEFINE USE_RX_LIBRARY}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
{$IFDEF USE_RX_LIBRARY}
  Placemnt,  // For RxLib component TFormStorage.
{$ENDIF}
  StdCtrls, ActnList, ComCtrls, Menus, ExtCtrls;

type
  TFormOnTheRun = class(TForm)
    LabelArguments: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ComboBoxArguments: TComboBox;
    OpenDialog1: TOpenDialog;
    CheckBoxPassArgumentsAsShortPathNames: TCheckBox;
    ComboBoxDoubleQuotes: TComboBox;
    ButtonArguments: TButton;
    CheckBoxPassArgumentsAtStartup: TCheckBox;
    CheckBoxMinimizeAfterPassingArguments: TCheckBox;
    ComboBoxWindow: TComboBox;
    ComboBoxCommand: TComboBox;
    ButtonBrowseForFile: TButton;
    ButtonGo: TButton;
    ActionList1: TActionList;
    ActionClose: TAction;
    FormStorage1: TFormStorage;
    Memo1: TMemo;
    ActionStatusBar: TAction;
    CheckBox1ArgumentOnly: TCheckBox;
    CheckBoxOptions: TCheckBox;
    PanelOptions: TPanel;
    PanelTop: TPanel;
    RadioGroupWhatIfDroppingArgumentsOnWindowFails: TRadioGroup;
    Label1: TLabel;
    PanelClient: TPanel;
    Panel1: TPanel;
    procedure ComboBoxCommandKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    function SendArgumentsToAnotherInstanceOfOnTheRun: boolean;
    procedure ComboBoxArgumentsChange(Sender: TObject);
    procedure CheckBoxPassArgumentsAsShortPathNamesClick(Sender: TObject);
    procedure ComboBoxDoubleQuotesChange(Sender: TObject);
    procedure ComboBoxWindowChange(Sender: TObject);
    procedure ComboBoxCommandChange(Sender: TObject);
    procedure ComboBoxArgumentsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ButtonBrowseForFileClick(Sender: TObject);
    procedure ButtonArgumentsClick(Sender: TObject);
    procedure ComboBoxWindowDropDown(Sender: TObject);
    procedure ButtonGoClick(Sender: TObject);
    procedure ActionCloseExecute(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ActionStatusBarExecute(Sender: TObject);
    procedure CheckBoxOptionsClick(Sender: TObject);
    procedure CheckBox1ArgumentOnlyClick(Sender: TObject);
  protected
    Procedure CreateParams( Var params: TCreateParams ); override;
  private
    FHeightOfOptions: Integer;
    FDefaultHeightOfStatusBar: Integer;
    FOverwriteStatusText: Boolean;
    procedure MinimizeThisApplication;
    procedure OnFirstIdleEventHandler(Sender: TObject; var Done: Boolean);
    procedure WMSysCommand(var Msg: TWMSysCommand); message WM_SYSCOMMAND;
    procedure WMCopyData(var msg: TWMCopyData); message WM_COPYDATA;
    procedure ToggleStatusBar;
    procedure ShowAboutBox;
    procedure ShowArgumentsInStatusBar;
    function PassArguments: boolean;
    procedure InsertSystemMenuItems(const hSystemMenu: HMENU);
    procedure EnableGo;
    procedure BrowseForFile;
    procedure EditArguments;
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure GoToTheWebsite;
  end;
  TCustomMenuItems = (TOGGLE_STATUS_BAR, ON_THE_WEB, SHOW_ABOUT_BOX);

var
  FormOnTheRun: TFormOnTheRun;

  // Fix the system menu at the Windows Taskbar if the following
  // variable is True.  (For any application made in Delphi 5-7, the
  // system menu that appears when right-clicking its taskbar button
  // is incomplete by default!  This fix was found on the internet by
  // lsp, and was previously marked as "Knispering" in the source files.)
  bFixSystemMenu: Boolean;

implementation

{$R *.DFM}

uses
  AboutBoxUnit,
  ArgumentsUnit,
  FileCtrl,
{$IFDEF USE_RX_LIBRARY}
  Rxconst,   // For RX_VERSION.
{$ENDIF}
  ShellApi,
  shlobj,
  TypInfo;

const
  Checks: array[Boolean] of DWORD = (MF_UNCHECKED, MF_CHECKED);

type
  TTextAndHandle = record
    Text: string;
    Handle: HWND;
  end;


// Drops the arguments on the window by posting a DROPFILES message.
// Returns true iff succesfully posted.  Unfortunately the window might
// still ignore the message, even if it is successfully posted.
function DropArgumentsOnWindow(
  const hWindow: HWND;
  const rArgumentList: TStringList): boolean;
var
  dwBytes: DWORD;
  hg: HGLOBAL;
  i: Integer;
  pc: PChar;
  pd: PDropFiles;
  strArgument: String;
begin
  Result := False;
  if (hWindow <> 0) and (rArgumentList <> nil) then
  begin
    // The number of bytes needed for the drop structure, including
    // the list of zero-terminated arguments.
    dwBytes := SizeOf(TDropFiles) + 1;
    for i := 0 to rArgumentList.Count - 1 do
      Inc(dwBytes, Length(rArgumentList[i]) + 1);

    hg := GlobalAlloc(GMEM_SHARE or GMEM_MOVEABLE or GMEM_ZEROINIT,
      dwBytes);
    if hg = 0 then
      Application.MessageBox(
        PChar('Failed to allocate ' + IntToStr(dwBytes) +
          ' bytes from the global heap.'#13#10 +
          SysErrorMessage( GetLastError() ) ),
        'Error preparing to drop arguments on window',
        MB_ICONEXCLAMATION)
    else
    begin
      pd := PDropFiles(GlobalLock(hg));
      if pd = nil then
        Application.MessageBox(
          PChar('Failed to lock a global memory object.'#13#10 +
            SysErrorMessage( GetLastError() ) ),
          'Error preparing to drop arguments on window',
          MB_ICONEXCLAMATION)
      else
      begin
        // Copy the arguments to the end of the drop structure.
        pd.pFiles := SizeOf(TDropFiles);
        pc := PChar(pd) + SizeOf(TDropFiles);
        for i := 0 to rArgumentList.Count - 1 do
        begin
          strArgument := rArgumentList[i];
          StrCopy(pc, PChar(strArgument));
          // Each argument is terminated by a zero byte.
          Inc(pc, Length(strArgument) + 1)
        end;
        GlobalUnlock(hg);
        // By now, HDROP(hg) is a nice drop handle.  Ready to be sent!
        // However, SendMessage didn't work, so let's post the drop!
        if PostMessage(hWindow, WM_DROPFILES, WPARAM(hg), 0) then
          Result := True
        else
          Application.MessageBox(
            PChar('Failed to post a message.'#13#10 +
              SysErrorMessage( GetLastError() ) ),
            'Error trying to drop arguments on window',
            MB_ICONEXCLAMATION)
      end
    end
  end
end;


function GetWindowsDirWithTrailingBackslash: string;
var
  uLength: UINT;
begin
  try
    SetLength(Result, MAX_PATH + 1);
    SetLastError(0);
    uLength := GetWindowsDirectory(PChar(Result), MAX_PATH);
    if (uLength > 0) and (GetLastError() = 0) then
    begin
      if uLength > MAX_PATH then
      begin
        // Result was not big enough to hold the path.
        SetLength(Result, uLength + 1);
        SetLastError(0);
        uLength := GetWindowsDirectory(PChar(Result), uLength);
        if (uLength > 0) and (GetLastError() = 0) then
          Result := IncludeTrailingBackslash( PChar(Result) )
        else
          // Second call to GetWindowsDirectory failed.
          Result := ''
      end
      else
        Result := IncludeTrailingBackslash( PChar(Result) )
    end
    else
      // First call to GetWindowsDirectory failed.
      Result := ''
  except
    Result := ''
  end;
end;


function MakeFileTypeFilter(
  // Filetype description, e.g. 'Text File'.
  const strFileTypeDescription: string;
  // File extension, e.g. '.txt'.
  const strFileExt: string):
  string;
begin
  result := strFileTypeDescription + ' (*' + strFileExt + ')|*' + strFileExt
end;

// Finds the first existing directory of the specified array.
function FindFirstExistingDirectory(
  const aDirectories: array of String):
  string;
var
  i: integer;
begin
   for i := 0 to High(aDirectories) do
   begin
     result := aDirectories[i];
     if DirectoryExists(result) then
       exit
   end;
   result := ''
end;


// Taken from the System unit: borland\delphi5\source\rtl\Sys\System.pas
function GetParamStr(P: PChar; var Param: string): PChar;
var
  Len: Integer;
  Buffer: array[0..4095] of Char;
begin
  while True do
  begin
    while (P[0] <> #0) and (P[0] <= ' ') do Inc(P);
    if (P[0] = '"') and (P[1] = '"') then Inc(P, 2) else Break;
  end;
  Len := 0;
  while (P[0] > ' ') and (Len < SizeOf(Buffer)) do
    if P[0] = '"' then
    begin
      Inc(P);
      while (P[0] <> #0) and (P[0] <> '"') do
      begin
        Buffer[Len] := P[0];
        Inc(Len);
        Inc(P);
      end;
      if P[0] <> #0 then Inc(P);
    end else
    begin
      Buffer[Len] := P[0];
      Inc(Len);
      Inc(P);
    end;
  SetString(Param, Buffer, Len);
  Result := P;
end;


// Puts s at the begin of the list. It adds s to the list if s is not
// already there.  If the list was at its maximum size before adding
// the string, the item at the end of the list will be removed.
// Returns true iff successful.
function PutStringAtBeginOfList(
  const rList: TStrings;
  const s: string;
  const iMaxSize: integer): Boolean;
var
  i: Integer;
begin
  Result := False;
  if s <> '' then
  begin
    rList.BeginUpdate();
    try
      i := rList.IndexOf(S);
      if i < 0 then
      begin
        if (iMaxSize > 0) and (rList.Count = iMaxSize) then
          rList.Delete(iMaxSize - 1);
        rList.Insert(0, S);
      end
      else
        // s was already in the list, at position i.  So move it
        // to the begin of the list!
        rList.Move(i, 0);

      Result := True
    finally
       rList.EndUpdate()
    end
  end
end;

// Puts s at the end of the list.
procedure PutStringAtEndOfList(
  const rList: TStrings;
  const s: string;
  const bOverwrite: Boolean);
var
  iCount: Integer;
begin
  iCount := rList.Count;
  if iCount = 0 then
    rList.Add(s)
  else
    if bOverwrite then
      rList[iCount - 1] := s
    else
      // Always overwrite an empty string.
      if rList[iCount - 1] = '' then
        rList[iCount - 1] := s
      else
        if rList[iCount - 1] <> s then
          rList.Add(s)
end;


// This is an EnumWindowsProc callback function to find a window whose
// window text (or caption) contains the user specified text.
// lParam should point to a TextAndHandle record.  The caller should
// put the search text into this record, and a handle to the window
// found so far (or 0, if no window was found yet).  The callback
// function will replace this handle by hwnd if the hwnd-window has a
// text that matches the search text more closely than the text of the
// window found so far.
function EnumProcToFindWindowByText(
  hwnd: HWND; lParam: LPARAM): BOOL; export; stdcall;
type
  TPTextAndHandle = ^TTextAndHandle;
var
  strText: string;
  iLength: Integer;
  pTextAndHandle: TPTextAndHandle;
begin
  pTextAndHandle := TPTextAndHandle(lParam);
  if pTextAndHandle = nil then
    // Invalid parameter, so return false to stop enumeration.
    Result := False
  else
  begin
    // Return true to continue enumeration.
    Result := True;

    if hwnd <> 0 then
    begin
      iLength := GetWindowTextLength(hwnd);
      SetLength(strText, iLength + 1);
      SetLastError(0);
      GetWindowText(hwnd, PChar(strText), Length(strText));
      if GetLastError() = 0 then
      begin
        strText := PChar(strText);
        if strText = pTextAndHandle.Text then
        begin
          pTextAndHandle.Handle := hwnd;
          // The window text matches exactly, so return false to
          // stop enumeration.
          Result := False
        end
        else
          if Pos(pTextAndHandle.Text, strText) > 0 then
          begin
            // The window text of hwnd contains the search text.
            // If it is a closer match than the previously found one
            // then put it into the record pointed to by lParam!
            if pTextAndHandle.Handle = 0 then
              pTextAndHandle.Handle := hwnd
            else
              if iLength < GetWindowTextLength(pTextAndHandle.Handle) then
                pTextAndHandle.Handle := hwnd
          end
      end
    end
  end
end;


// This is an EnumWindowsProc callback function to add the window text
// (or caption) of all open windows to a list of strings.  It will not
// add any empty string or duplicate to the list.
function EnumProcToAddWindowTextToList(
  hwnd: HWND; lParam: LPARAM): BOOL; export; stdcall;
var
  strText: string;
  rList: TStrings;
begin
  // Return false unless lParam points to a valid list of strings.
  Result := False;
  if lParam <> 0 then
    if TObject(lParam) is TStrings then
    begin
      rList := TStrings(lParam);
      if rList <> nil then
      begin
        // Return True to continue enumeration.
        Result := True;
        SetLength(strText, GetWindowTextLength(hwnd) + 1);
        SetLastError(0);
        GetWindowText(hwnd, PChar(strText), Length(strText));
        if GetLastError() = 0 then
        begin
          strText := PChar(strText);
          if strText <> '' then
            if rList.IndexOf(strText) < 0 then
              rList.Add(strText)
        end
      end
    end
end;


// Deletes the pair of double-quotes that is enclosing the sting.
function DeleteEnclosingDoubleQuotes(const s: string): string;
begin
  if Length(s) > 1 then
    if (s[1] = '"') and (s[Length(s)] = '"') then
      result := Copy(s, 2, Length(s) - 2)
    else
      result := s
  else
    result := s
end;

// Returns the double-quoted version of a string.
function DoubleQuotedStr(const s: string): string;
begin
  if Pos('"', s) = 0 then
    // The string will be enclosed by double-quotes:
    result := '"' + s + '"'
  else
    // The string already has some double-quote(s)!
    result := s
end;


function ConvertToShortPathName(const strArgument: string): string;
var
  Handle: THandle;
  FindData: TWin32FindData;
  szShortPathName: array[0..MAX_PATH] of char;
  bDoubleQuotes: Boolean;
begin
  bDoubleQuotes := False;
  Result := strArgument;
  if Length(Result) > 1 then
  begin
    if (Result[1] = '"') and (Result[Length(Result)] = '"') then
    begin
      Result := Copy(Result, 2, Length(Result) - 2);
      bDoubleQuotes := True
    end;
    Handle := FindFirstFile(PChar(Result), FindData);
    if Handle <> INVALID_HANDLE_VALUE then
    begin
      FillChar(szShortPathName, SizeOf(szShortPathName), 0);
      if GetShortPathName(PChar(Result), szShortPathName, MAX_PATH) <> 0 then
        if String(szShortPathName) <> '' then
          Result := String(szShortPathName);
      Windows.FindClose(Handle)
    end;
    if bDoubleQuotes then
      Result := '"' + Result + '"'
  end
end;


function ConvertToShortPathNames(const strArguments: string): string;
var
  strParam: string;
  p, pPrevious: PChar;
  strShortPathName: String;
  iPosOfParamInResult: integer;
begin
  pPrevious := PChar(strArguments);
  p := GetParamStr(pPrevious, strParam);
  result := strArguments;

  while (p <> nil) and (strParam <> '') do
  begin
    strShortPathName := ConvertToShortPathName(strParam);
    if (strShortPathName <> '') and (strShortPathName <> strParam) then
    begin
      iPosOfParamInResult :=
        Pos(strParam, pPrevious) + Length(Result) - Length(pPrevious);
      Result :=
        Copy(result, 1, iPosOfParamInResult - 1) +
        strShortPathName +
        Copy(result, iPosOfParamInResult + Length(strParam), MaxInt)
    end;
    pPrevious := p;
    p := GetParamStr(pPrevious, strParam)
  end
end;


function RemoveDoubleQuotesFromArguments(const strArguments: string): string;
var
  strParam: string;
  p, pPrevious: PChar;
  iPosOfParamInResult: integer;
begin
  pPrevious := PChar(strArguments);
  p := GetParamStr(pPrevious, strParam);
  result := strArguments;

  while (p <> nil) and (strParam <> '') do
  begin
    iPosOfParamInResult := Pos(strParam, pPrevious) + Length(result) - Length(pPrevious);

    if iPosOfParamInResult > 1 then
      if result[iPosOfParamInResult - 1] = '"' then
        if iPosOfParamInResult + Length(strParam) <= Length(result) then
          if result[iPosOfParamInResult + Length(strParam)] = '"' then
            if Pos(' ', strParam) = 0 then
              result :=
                Copy(result, 1, iPosOfParamInResult - 2) +
                strParam +
                Copy(result, iPosOfParamInResult + Length(strParam) + 1, MaxInt);
    pPrevious := p;
    p := GetParamStr(pPrevious, strParam)
  end
end;


function AddDoubleQuotesToArguments(const strArguments: string): string;
var
  strParam: string;
  p, pPrevious: PChar;
  iPosOfParamInResult: integer;
begin
  pPrevious := PChar(strArguments);
  p := GetParamStr(pPrevious, strParam);
  result := strArguments;

  while (p <> nil) and (strParam <> '') do
  begin
    iPosOfParamInResult := Pos(strParam, pPrevious) + Length(result) - Length(pPrevious);

    if iPosOfParamInResult > 1 then
      if result[iPosOfParamInResult - 1] = '"' then
        if iPosOfParamInResult + Length(strParam) <= Length(result) then
          if result[iPosOfParamInResult + Length(strParam)] = '"' then
          begin
            pPrevious := p;
            p := GetParamStr(pPrevious, strParam);
            Continue;
          end;
    result :=
      Copy(result, 1, iPosOfParamInResult - 1) +
      DoubleQuotedStr(strParam) +
      Copy(result, iPosOfParamInResult + Length(strParam), MaxInt);
    pPrevious := p;
    p := GetParamStr(pPrevious, strParam)
  end
end;


procedure AddDoubleQuotesToStringsInList(const rList: TStrings);
var
  i: Integer;
  s: string;
begin
  for i := 0 to rList.Count - 1 do
  begin
    s := rList[i];
    if s = '' then
      rList[i] := '""'
    else
      if (s[1] = '"') and (s[Length(s)] = '"') then
        rList[i] := s
      else
        rList[i] := '"' + s + '"'
  end
end;


procedure RemoveDoubleQuotesFromStringsInList(const rList: TStrings);
var
  i: Integer;
begin
  for i := 0 to rList.Count - 1 do
    rList[i] := DeleteEnclosingDoubleQuotes(rList[i])
end;


procedure RemoveEmptyStringsFromList(const rList: TStrings);
var
  i: Integer;
begin
  for i := rList.Count - 1 downto 0 do
    if rList[i] = '' then
      rList.Delete(i)
end;


procedure ScrollMemoDown(const rMemo: TMemo);
begin
  SendMessage(rMemo.Handle, WM_VSCROLL,
    MakeLong(SB_THUMBPOSITION, rMemo.Lines.Count - 1), 0 );
end;


////////////////////////////////////////////////////////////////////////////////
// Helper methods


procedure TFormOnTheRun.MinimizeThisApplication;
begin
  if bFixSystemMenu then
    ShowWindow(Handle, SW_MINIMIZE)
  else
    Application.Minimize();
    // Or (consider): ShowWindow(Application.Handle, SW_MINIMIZE);
end;


procedure TFormOnTheRun.InsertSystemMenuItems(const hSystemMenu: HMENU);
begin
  InsertMenu(
    hSystemMenu,
    SC_CLOSE,
    Checks[Panel1.Height = FDefaultHeightOfStatusBar] + MF_STRING,
    WM_APP + Ord(TOGGLE_STATUS_BAR),
    PChar('Status &Bar'#09 + ShortCutToText(ActionStatusBar.ShortCut)));
  InsertMenu(
    hSystemMenu,
    SC_CLOSE,
    MF_STRING,
    WM_APP + Ord(ON_THE_WEB),
    PChar('&On The Run - On The Web'));
  InsertMenu(
    hSystemMenu,
    SC_CLOSE,
    MF_STRING,
    WM_APP + Ord(SHOW_ABOUT_BOX),
    PChar('&About ' + Application.Title + '...'));
  InsertMenu(
    hSystemMenu,
    SC_CLOSE,
    MF_SEPARATOR,
    0,
    '')
end;


function TFormOnTheRun.SendArgumentsToAnotherInstanceOfOnTheRun: Boolean;
var
  cd: TCopyDataStruct;
  strArguments, strFirstParam: string;
  hDestination: HWnd;
  strCaption: string;
begin
  strCaption :=
    Application.Title + ' - Passing your arguments to a window or a command';
  hDestination := FindWindow(
    PChar(string(TFormOnTheRun.ClassName())),
    PChar(strCaption) );

  Result := HDestination <> 0;
  if Result then
  begin
    // WM_COPYDATA stuff inspired by the Japanese website
    // http://www2.big.or.jp/~osamu/Delphi/Tips/key.cgi?key=32
    cd.dwData := 0;
    strArguments := Trim(GetParamStr(GetCommandLine(), strFirstParam));
    cd.cbData := Length(strArguments)+1;
    cd.lpData := StrAlloc(cd.cbData);
    StrCopy(cd.lpData, PChar(strArguments));
    SendMessage(hDestination, WM_COPYDATA, WPARAM(Handle), LPARAM(@cd));
    StrDispose(cd.lpData)
  end
  else
    Caption := strCaption
end;


// This is in fact the "main function" of OnTheRun.
function TFormOnTheRun.PassArguments: boolean;
var
  strCommand: string;
  strStatus: string;
  strArguments: string;
  ArgumentList: TStringList;
  strText: string;
  TextAndHandle: TTextAndHandle;
begin
  strStatus := '';
  Result := False;
  strArguments := ComboBoxArguments.Text;
  ArgumentList := TStringList.Create();
  try
    if CheckBox1ArgumentOnly.Checked then
    begin
      if CheckBoxPassArgumentsAsShortPathNames.Checked then
        strArguments := ConvertToShortPathName(strArguments);
      case ComboBoxDoubleQuotes.ItemIndex of
      1: strArguments := DeleteEnclosingDoubleQuotes(strArguments);
      2: strArguments := DoubleQuotedStr(strArguments)
      end;
      ArgumentList.Add(strArguments);
    end
    else
    begin
      if CheckBoxPassArgumentsAsShortPathNames.Checked then
        strArguments := ConvertToShortPathNames(strArguments);
      AddSpaceSeparatedArgumentsToList(strArguments, ArgumentList);
      case ComboBoxDoubleQuotes.ItemIndex of
        1:  // Remove double quotes from arguments
        begin
          strArguments := RemoveDoubleQuotesFromArguments(strArguments);
          RemoveDoubleQuotesFromStringsInList(ArgumentList)
        end;
        2:  // Add double quotes to arguments
        begin
          strArguments := AddDoubleQuotesToArguments(strArguments);
          AddDoubleQuotesToStringsInList(ArgumentList)
        end
      end;
      RemoveEmptyStringsFromList(ArgumentList);
    end;

    TextAndHandle.Text := ComboBoxWindow.Text;
    if TextAndHandle.Text = '' then
      strStatus := 'There is no window specified'
    else
    begin
      TextAndHandle.Handle := 0;
      // Handle will be set to a window whose window text contains the
      // text that was specified in the Window combo box.
      EnumWindows(@EnumProcToFindWindowByText, LPARAM(@TextAndHandle));

      if TextAndHandle.Handle = 0 then
        strStatus :=
          'Cannot not find window ' + DoubleQuotedStr(TextAndHandle.Text)
      else
      begin
        SetLength(strText, GetWindowTextLength(TextAndHandle.Handle) + 1);
        GetWindowText(TextAndHandle.Handle,
          PChar(strText), Length(strText));
        strText := PChar(strText);
        if DropArgumentsOnWindow(TextAndHandle.Handle, ArgumentList) then
        begin
          strStatus := 'Arguments dropped on window ' +
            DoubleQuotedStr(strText);
          Result := True
        end
        else
          strStatus := 'Failed to drop arguments on window ' +
            DoubleQuotedStr(strText);
      end
    end;
    if not Result then
    begin
      if strStatus = '' then
        // I don't know what might have gone wrong in this case.
        strStatus := 'Somehow failed to drop arguments on window';

      case RadioGroupWhatIfDroppingArgumentsOnWindowFails.ItemIndex of
      0:  // Run command, passing the arguments with the command
        FreeAndNil(ArgumentList);
      1:  // Run command, then try to drop arguments on window again
        strArguments := ''
      end;
      strCommand :=
        DeleteEnclosingDoubleQuotes(Trim(ComboBoxCommand.Text));
      if strCommand = '' then
        strStatus := strStatus +
          ' and there is no command specified either!'
      else
      begin
        // If ShellExecute fails, its return value is less than or equal to 32.
        if ShellExecute(0, 'open',
          PChar(strCommand), PChar(strArguments), nil, SW_SHOWNORMAL) <= 32 then
        begin
          strStatus := strStatus +
            ', cannot run command ' + DoubleQuotedStr(strCommand) + ' either!';
          Application.MessageBox(
            PChar(
              'Failed to run ' + DoubleQuotedStr(strCommand) + #13#10 +
               SysErrorMessage( GetLastError() ) ),
            'On The Runtime Error',
            MB_ICONEXCLAMATION)
        end
        else
        // ShellExecute succeeded
        begin
          if ArgumentList = nil then
            strStatus :=
              'Arguments are passed to command ' + DoubleQuotedStr(strCommand)
          else
          begin
            Sleep(1000);
            TextAndHandle.Handle := 0;
            // Handle will be set to a window whose window text contains the
            // text that was specified in the Window combo box.
            EnumWindows(@EnumProcToFindWindowByText, LPARAM(@TextAndHandle));

            if TextAndHandle.Handle = 0 then
              strStatus :=
                'Cannot not find window ' + DoubleQuotedStr(TextAndHandle.Text)
            else
            begin
              SetLength(strText, GetWindowTextLength(TextAndHandle.Handle) + 1);
              GetWindowText(TextAndHandle.Handle,
                PChar(strText), Length(strText));
              strText := PChar(strText);
              if DropArgumentsOnWindow(TextAndHandle.Handle, ArgumentList) then
              begin
                strStatus := 'Arguments dropped on window ' +
                  DoubleQuotedStr(strText);
                Result := True
              end
              else
                strStatus := 'Failed to drop arguments on window ' +
                  DoubleQuotedStr(TextAndHandle.Text);
            end
          end
        end;
        if PutStringAtBeginOfList(
            ComboBoxCommand.Items, strCommand, 10) then
          ComboBoxCommand.ItemIndex := 0;

        Result := True
    end
  end
  finally
    ArgumentList.Free()
  end;
  PutStringAtEndOfList(Memo1.Lines, strStatus, FOverwriteStatusText);
  FOverwriteStatusText := False;
  ScrollMemoDown(Memo1);

  if Result then
    if PutStringAtBeginOfList(
        ComboBoxArguments.Items, ComboBoxArguments.Text, 10) then
      ComboBoxArguments.ItemIndex := 0
end;


procedure TFormOnTheRun.ToggleStatusBar;
begin
  if Panel1.Height = FDefaultHeightOfStatusBar then
  begin
    Height := Height + 1 - FDefaultHeightOfStatusBar;
  end
  else
  begin
    // Doing it twice seems to work better!
    Height := Height + FDefaultHeightOfStatusBar - Panel1.Height;
    Height := Height + FDefaultHeightOfStatusBar - Panel1.Height;
  end;

  PutStringAtEndOfList(Memo1.Lines, '', FOverwriteStatusText);
  ScrollMemoDown(Memo1);

  CheckMenuItem(
    GetSystemMenu(Handle, false),
    WM_APP + Ord(TOGGLE_STATUS_BAR),
    MF_BYCOMMAND or Checks[Panel1.Height = FDefaultHeightOfStatusBar] );
  Memo1.TabStop := Memo1.Height > 1
end;


procedure TFormOnTheRun.ShowAboutBox;
begin
  try
    AboutBox := TAboutBox.Create(nil);

{$IFDEF USE_RX_LIBRARY}
    AboutBox.AdditionalVersionInfo :=
      'Used Delphi component library:'#13#10 +
      '  RXLib ' +
      IntToStr(RX_VERSION div $10000) + '.' +
      IntToStr(RX_VERSION mod $10000) +
      ' by Fedor Kozhevnikov, Igor Pavluk, and Serge Korolev'#13#10
{$IFDEF VER140}
      + '  RX Library port to Delphi 6 from http://www.oxygensoftware.com'#13#10
{$ENDIF}
{$IFDEF VER150}
      + '  RX Library port to Delphi 7 from http://www.oxygensoftware.com'#13#10
{$ENDIF}
      + 'Settings are stored at ' +
      DoubleQuotedStr(GetWindowsDirWithTrailingBackslash() + FormStorage1.IniFileName)
      ;
{$ENDIF}
    AboutBox.ShowModal();
  finally
    FreeAndNil(AboutBox)
  end
end;


procedure TFormOnTheRun.GoToTheWebsite;
begin
  ComboBoxWindow.Text := '';
  ComboBoxArguments.Text := '';
  ComboBoxCommand.Text := 'http://www.xs4all.nl/~nd/dekkerware/ontherun/';
  ButtonGo.Enabled := True;
  ButtonGo.Click()
end;


procedure TFormOnTheRun.ShowArgumentsInStatusBar;
var
  strStatus: string;
  strArguments: string;
begin
  strArguments := ComboBoxArguments.Text;

  if TrimLeft(strArguments) = '' then
    strStatus := ''
  else
  begin
    if CheckBox1ArgumentOnly.Checked then
    begin
      if CheckBoxPassArgumentsAsShortPathNames.Checked then
        strArguments := ConvertToShortPathName(strArguments);

      case ComboBoxDoubleQuotes.ItemIndex of
      1: strArguments := DeleteEnclosingDoubleQuotes(strArguments);
      2: strArguments := DoubleQuotedStr(strArguments)
      end;
      strStatus := 'Argument: ' + strArguments
    end
    else
    begin
      if CheckBoxPassArgumentsAsShortPathNames.Checked then
        strArguments := ConvertToShortPathNames(strArguments);

      case ComboBoxDoubleQuotes.ItemIndex of
      1: strArguments := RemoveDoubleQuotesFromArguments(strArguments);
      2: strArguments := AddDoubleQuotesToArguments(strArguments)
      end;
      strStatus := 'Arguments: ' + strArguments
    end
  end;

  PutStringAtEndOfList(Memo1.Lines, strStatus, FOverwriteStatusText);
  FOverwriteStatusText := False;
  ScrollMemoDown(Memo1);
end;


procedure TFormOnTheRun.EnableGo;
begin
  ButtonGo.Enabled :=
    (ComboBoxWindow.Text <> '') or
    (ComboBoxCommand.Text <> '')
end;


procedure TFormOnTheRun.BrowseForFile;
var
  strCommand, strFileName: string;
begin
  strCommand :=
    DeleteEnclosingDoubleQuotes(Trim(ComboBoxCommand.Text));
  strFileName := ExtractFileName(strCommand);
  if Pos('"', strFileName) = 0 then
    OpenDialog1.FileName := strFileName
  else
    OpenDialog1.FileName := '';
  OpenDialog1.Title := 'What Do You Want To Run?';
  OpenDialog1.Filter :=
    MakeFileTypeFilter('Application', '.exe') + '|' +
    'All files (*.*)|*.*';

  OpenDialog1.InitialDir := FindFirstExistingDirectory([
    ExtractFilePath(strCommand),
    'C:\Program Files\',
    'D:\Program Files\',
    'E:\Program Files\']);

  if OpenDialog1.Execute() then
  begin
    ComboBoxCommand.Text := OpenDialog1.FileName;
    EnableGo()
  end
end;


procedure TFormOnTheRun.EditArguments;
var
  iModalResult: integer;
begin
  if FormArguments = nil then
    FormArguments := TFormArguments.Create(self);

  if CheckBox1ArgumentOnly.Checked then
    FormArguments.Caption := 'Argument'
  else
    FormArguments.Caption := 'Arguments';

  FormArguments.Arguments := ComboBoxArguments.Text;

  iModalResult := FormArguments.ShowModal();

  if iModalResult = mrOK then
    ComboBoxArguments.Text := FormArguments.Arguments
end;


////////////////////////////////////////////////////////////////////////////////
// Event handlers



procedure TFormOnTheRun.OnFirstIdleEventHandler(Sender: TObject;
  var Done: Boolean);
var
  strFirstParam: string;
  bShowOptions: Boolean;
begin
  Application.OnIdle := nil;

  if bFixSystemMenu then
    Application.MainForm.Show;

  // Because now the mainform is shown, FormStorage1 has by now restored
  // the property values from the components.
  bShowOptions := CheckboxOptions.Checked;

  InsertSystemMenuItems( GetSystemMenu(Handle, false) );

  if ParamCount() > 0 then
    ComboBoxArguments.Text := Trim(GetParamStr(GetCommandLine(), strFirstParam));

  if not bShowOptions then
    PanelOptions.Height := 1;
  PanelOptions.Enabled := bShowOptions;

  // It seems like MinHeight should be 0 in the DFM-file, otherwise a
  // hidden Panel1 (height <= 1) gets restored incorrectly.
  Panel1.Constraints.MinHeight := 1;

  Memo1.TabStop := Memo1.Height > 1;

  EnumWindows(@EnumProcToAddWindowTextToList, LPARAM(ComboBoxWindow.Items));

  if CheckBoxPassArgumentsAtStartup.Checked then
    if PassArguments() then
      if CheckBoxMinimizeAfterPassingArguments.Checked then
        MinimizeThisApplication();

  CheckBoxOptions.OnClick := CheckBoxOptionsClick;
  OnResize := FormResize;
end;


procedure TFormOnTheRun.WMSysCommand(var Msg: TWMSysCommand);
begin
  case Msg.CmdType of
    WM_APP + Ord(TOGGLE_STATUS_BAR):
      ToggleStatusBar();
    WM_APP + Ord(SHOW_ABOUT_BOX):
      ShowAboutBox();
    WM_APP + Ord(ON_THE_WEB):
      GoToTheWebsite();
  else
  if bFixSystemMenu then
    case Msg.CmdType of
      SC_MINIMIZE:
      begin
        ShowWindow( handle, SW_MINIMIZE );
        msg.result := 0;
      end;
      SC_RESTORE:
      begin
        ShowWindow( handle, SW_RESTORE );
        msg.result := 0;
      end
    else
      inherited
    end
  else
    inherited
  end
end;


// WMCopyData stuff from http://www2.big.or.jp/~osamu/Delphi/Tips/key.cgi?key=32
procedure TFormOnTheRun.WMCopyData(var msg: TWMCopyData);
var
  bRestore: boolean;
begin
  bRestore := true;
  if msg.CopyDataStruct.dwData = 0 then
  begin
    PutStringAtEndOfList(Memo1.Lines, '', FOverwriteStatusText);
    ScrollMemoDown(Memo1);

    ComboBoxArguments.Text := PChar(msg.CopyDataStruct.lpData);

    if CheckBoxPassArgumentsAtStartup.Checked then
      if PassArguments() then
        if CheckBoxMinimizeAfterPassingArguments.Checked then
          bRestore := false;

    if bRestore then
    begin
      if bFixSystemMenu then
        ShowWindow(Handle, SW_RESTORE)
      else
        Application.Restore();
        // Or (consider): ShowWindow(Application.Handle, SW_RESTORE);

      SetForegroundWindow(Handle)
    end
    else
      MinimizeThisApplication()
  end
  else
    inherited;
end;



procedure TFormOnTheRun.ComboBoxCommandKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  Msg: TMsg;
begin
  if Key = VK_RETURN then
  begin
    if not ComboBoxCommand.DroppedDown then
    begin
      // The following line disables the beep, as described in:
      // "...disable the Enter Key Beep in a Edit Box?", by Thomas Stutz,
      // at http://www.swissdelphicenter.ch/en/showcode.php?id=432
      PeekMessage(Msg, 0, WM_CHAR, WM_CHAR, PM_REMOVE);
      BrowseForFile()
    end
  end
end;


procedure TFormOnTheRun.ComboBoxArgumentsKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  Msg: TMsg;
begin
  if Key = VK_RETURN then
  begin
    if not ComboBoxArguments.DroppedDown then
    begin
      // The following line disables the beep, as described in:
      // "...disable the Enter Key Beep in a Edit Box?", by Thomas Stutz,
      // at http://www.swissdelphicenter.ch/en/showcode.php?id=432
      PeekMessage(Msg, 0, WM_SYSCHAR, WM_SYSCHAR, PM_REMOVE);
      PeekMessage(Msg, 0, WM_CHAR, WM_CHAR, PM_REMOVE);
      EditArguments()
    end
  end
end;


procedure TFormOnTheRun.FormCreate(Sender: TObject);
begin
  OnResize := nil;
  CheckBoxOptions.OnClick := nil;
  FOverwriteStatusText := False;
  if bFixSystemMenu then
    SetWindowLong( Application.handle,
                   GWL_EXSTYLE,
                   GetWindowLong( application.handle, GWL_EXSTYLE ) and
                     not WS_EX_APPWINDOW or WS_EX_TOOLWINDOW);

  Constraints.MinWidth := Width;

  // Unlike Delphi 6 and 7, Delphi 5 does not support setting
  // ItemIndex in Object Inspector, so let's do it down here!
  ComboBoxDoubleQuotes.ItemIndex := 0;

  FDefaultHeightOfStatusBar := Panel1.Height;
  FHeightOfOptions := PanelOptions.Height;

  DragAcceptFiles(Handle, True);
  Application.OnIdle := OnFirstIdleEventHandler
end;


procedure TFormOnTheRun.ComboBoxArgumentsChange(Sender: TObject);
begin
  ShowArgumentsInStatusBar()
end;

procedure TFormOnTheRun.CheckBoxPassArgumentsAsShortPathNamesClick(
  Sender: TObject);
begin
  ShowArgumentsInStatusBar()
end;

procedure TFormOnTheRun.ComboBoxDoubleQuotesChange(Sender: TObject);
begin
  ShowArgumentsInStatusBar()
end;

procedure TFormOnTheRun.ComboBoxWindowChange(Sender: TObject);
begin
  PutStringAtEndOfList(Memo1.Lines, '', FOverwriteStatusText);
  ScrollMemoDown(Memo1);
  EnableGo()
end;


procedure TFormOnTheRun.ComboBoxCommandChange(Sender: TObject);
var
  strFile: string;
  strStatus: string;
begin
  strFile := ComboBoxCommand.Text;
  if TrimLeft(strFile) = '' then
    strStatus := ''
  else
    if FileExists(DeleteEnclosingDoubleQuotes(Trim(strFile))) then
      strStatus := ''
    else
      strStatus := 'File does not exist: ' + strFile;
  PutStringAtEndOfList(Memo1.Lines, strStatus, FOverwriteStatusText);
  FOverwriteStatusText := True;
  ScrollMemoDown(Memo1);
  EnableGo()
end;


procedure TFormOnTheRun.ButtonBrowseForFileClick(Sender: TObject);
begin
  BrowseForFile()
end;

procedure TFormOnTheRun.ButtonArgumentsClick(Sender: TObject);
begin
  EditArguments()
end;

procedure TFormOnTheRun.CreateParams(var params: TCreateParams);
begin
  inherited CreateParams( params );

  if bFixSystemMenu then
    params.ExStyle := params.ExStyle  or WS_EX_APPWINDOW;
end;

procedure TFormOnTheRun.ComboBoxWindowDropDown(Sender: TObject);
begin
  ComboBoxWindow.Items.Clear();
  EnumWindows(@EnumProcToAddWindowTextToList, LPARAM(ComboBoxWindow.Items));
end;

procedure TFormOnTheRun.ButtonGoClick(Sender: TObject);
begin
  if PassArguments() then
    if CheckBoxMinimizeAfterPassingArguments.Checked then
      MinimizeThisApplication()
end;

procedure TFormOnTheRun.ActionCloseExecute(Sender: TObject);
begin
  Close()
end;

function GetDroppedFilenames(const hd: HDROP): string;
var
  strFilename: string;
  uFile: UINT;
  uCountFiles: UINT;
  uFilenameLength: UINT;
begin
  uCountFiles := DragQueryFile(hd, $FFFFFFFF, nil, 0);
  if uCountFiles = 0 then
    Result := ''
  else
  begin
    uFilenameLength := DragQueryFile(hd, 0, nil, 0);
    SetLength(strFilename, uFilenameLength + 1);
    DragQueryFile(hd, 0, PChar(strFilename), Length(strFilename) );
    strFilename := PChar(strFilename);

    if (Pos(' ', strFilename) <> 0) and (Pos('"', strFilename) = 0) then
      Result := '"' + strFilename +'"'
    else
      Result := strFilename;

    for uFile := 1 to uCountFiles - 1 do
    begin
      uFilenameLength := DragQueryFile(hd, uFile, nil, 0);
      SetLength(strFilename, uFilenameLength + 1);
      DragQueryFile(hd, uFile, PChar(strFilename), Length(strFilename));
      strFilename := PChar(strFilename);
      if (Pos(' ', strFilename) <> 0) and (Pos('"', strFilename) = 0) then
        strFilename :=  '"' + strFilename + '"';
      Result := Result + ' ' + strFilename
    end
  end
end;


function GetFirstDroppedFilename(const hd: HDROP): string;
var
  uCountFiles: UINT;
  uFilenameLength: UINT;
begin
  uCountFiles := DragQueryFile(hd, $FFFFFFFF, nil, 0);
  if uCountFiles = 0 then
    Result := ''
  else
  begin
    uFilenameLength := DragQueryFile(hd, 0, nil, 0);
    SetLength(Result, uFilenameLength + 1);
    DragQueryFile(hd, 0, PChar(Result), Length(Result) );
    Result := PChar(Result);
  end;
end;


procedure TFormOnTheRun.WMDropFiles(var Msg: TWMDropFiles);
var
  pt: TPoint;
  h: HWND;
begin
  inherited;
  try
    if DragQueryPoint(Msg.Drop, pt) then
    begin
      h := ChildWindowFromPoint(Handle, pt);

      if (h = ComboBoxCommand.Handle) and
        (DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0) = 1) then
      begin
        ComboBoxCommand.Text := GetFirstDroppedFilename(Msg.Drop);
        EnableGo()
      end
      else
        ComboBoxArguments.Text := GetDroppedFilenames(Msg.Drop)
    end
    else
      ComboBoxArguments.Text := GetDroppedFilenames(Msg.Drop)
  finally
    DragFinish(Msg.Drop)
  end
end;


procedure TFormOnTheRun.FormResize(Sender: TObject);
begin
  CheckMenuItem(
    GetSystemMenu(Handle, false),
    WM_APP + Ord(TOGGLE_STATUS_BAR),
    MF_BYCOMMAND or Checks[Panel1.Height = FDefaultHeightOfStatusBar] );
  Memo1.TabStop := Memo1.Height > 1
end;

procedure TFormOnTheRun.ActionStatusBarExecute(Sender: TObject);
begin
  ToggleStatusBar()
end;


procedure TFormOnTheRun.CheckBox1ArgumentOnlyClick(Sender: TObject);
begin
  if CheckBox1ArgumentOnly.Checked then
  begin
    LabelArguments.Caption := '&Argument:'
  end
  else
  begin
    LabelArguments.Caption := '&Arguments:'
  end;
  ShowArgumentsInStatusBar()
end;


procedure TFormOnTheRun.CheckBoxOptionsClick(Sender: TObject);
var
  iOldHeight: Integer;
begin
  iOldHeight := PanelOptions.Height;
  if CheckboxOptions.Checked then
  begin
    Height := Height + FHeightOfOptions - iOldHeight;
    PanelOptions.Height := FHeightOfOptions;
    PanelOptions.Enabled := True
  end
  else
  begin
    PanelOptions.Height := 1;
    Height := Height + 1 - iOldHeight;
    PanelOptions.Enabled := False
  end;
  PutStringAtEndOfList(Memo1.Lines, '', FOverwriteStatusText);
  ScrollMemoDown(Memo1)
end;


initialization
  bFixSystemMenu :=
    (Pos('INCOMPLETE_TASKBAR_MENU', Uppercase(ParamStr(0)) ) = 0) and
    (Pos('INCOMPLETE_TASKBAR_MENU', Uppercase(GetCurrentDir()) ) = 0)

end.
